<?php

declare(strict_types=1);

namespace Drupal\Tests\presigned_url\Kernel;

use Drupal\Core\Site\Settings;
use Drupal\KernelTests\KernelTestBase;
use Drupal\presigned_url\Signature\PresignedUrlSignatureGenerator;

/**
 * Tests the presigned URL signature generation.
 *
 * @group presigned_url
 * @coversDefaultClass \Drupal\presigned_url\Signature\PresignedUrlSignatureGenerator
 */
class PresignedUrlSignatureGeneratorTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['presigned_url'];

  /**
   * The signature generator service.
   */
  protected PresignedUrlSignatureGenerator $signatureGenerator;

  /**
   * The test hash salt.
   */
  protected string $hashSalt = 'test-hash-salt';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create a new settings object with a known hash salt
    // for predictable testing.
    new Settings([
      'hash_salt' => $this->hashSalt,
    ]);

    $this->signatureGenerator = new PresignedUrlSignatureGenerator(\Drupal::service('settings'));
  }

  /**
   * Tests that signatures are generated correctly.
   *
   * @covers ::generateSignature
   */
  public function testGenerateSignature(): void {
    $host = 'example.com';
    $uri = '/files/test.pdf';
    $date = '2024-03-20';
    $expires = 3600;
    $algorithm = 'sha256';

    $signature = $this->signatureGenerator->generateSignature(
      $host,
      $uri,
      $date,
      $expires,
      $algorithm
    );

    // Test that the signature is a valid SHA-256 hash (64 characters long)
    $this->assertEquals(64, strlen($signature));
    $this->assertTrue(ctype_xdigit($signature));

    // Test that the same input produces the same signature.
    $signature2 = $this->signatureGenerator->generateSignature(
      $host,
      $uri,
      $date,
      $expires,
      $algorithm
    );
    $this->assertEquals($signature, $signature2);
  }

  /**
   * Tests that different inputs produce different signatures.
   *
   * @covers ::generateSignature
   */
  public function testDifferentInputsProduceDifferentSignatures(): void {
    $baseHost = 'example.com';
    $baseUri = '/files/test.pdf';
    $baseDate = '2024-03-20';
    $baseExpires = 3600;
    $baseAlgorithm = 'sha256';

    $baseSignature = $this->signatureGenerator->generateSignature(
      $baseHost,
      $baseUri,
      $baseDate,
      $baseExpires,
      $baseAlgorithm
    );

    // Test different host.
    $differentHostSignature = $this->signatureGenerator->generateSignature(
      'different.com',
      $baseUri,
      $baseDate,
      $baseExpires,
      $baseAlgorithm
    );
    $this->assertNotEquals($baseSignature, $differentHostSignature);

    // Test different URI.
    $differentUriSignature = $this->signatureGenerator->generateSignature(
      $baseHost,
      '/files/other.pdf',
      $baseDate,
      $baseExpires,
      $baseAlgorithm
    );
    $this->assertNotEquals($baseSignature, $differentUriSignature);

    // Test different date.
    $differentDateSignature = $this->signatureGenerator->generateSignature(
      $baseHost,
      $baseUri,
      '2024-03-21',
      $baseExpires,
      $baseAlgorithm
    );
    $this->assertNotEquals($baseSignature, $differentDateSignature);

    // Test different expiry.
    $differentExpirySignature = $this->signatureGenerator->generateSignature(
      $baseHost,
      $baseUri,
      $baseDate,
      7200,
      $baseAlgorithm
    );
    $this->assertNotEquals($baseSignature, $differentExpirySignature);

    // Test different algorithm.
    $differentAlgorithmSignature = $this->signatureGenerator->generateSignature(
      $baseHost,
      $baseUri,
      $baseDate,
      $baseExpires,
      'sha512'
    );
    $this->assertNotEquals($baseSignature, $differentAlgorithmSignature);
  }

  /**
   * Tests that different hash salts produce different signatures.
   *
   * @covers ::generateSignature
   */
  public function testDifferentHashSaltProducesDifferentSignature(): void {
    $host = 'example.com';
    $uri = '/files/test.pdf';
    $date = '2024-03-20';
    $expires = 3600;
    $algorithm = 'sha256';

    $signature1 = $this->signatureGenerator->generateSignature(
      $host,
      $uri,
      $date,
      $expires,
      $algorithm
    );

    // Create a new settings object with a different hash salt.
    new Settings([
      'hash_salt' => 'different-hash-salt',
    ]);

    $newSignatureGenerator = new PresignedUrlSignatureGenerator(\Drupal::service('settings'));
    $signature2 = $newSignatureGenerator->generateSignature(
      $host,
      $uri,
      $date,
      $expires,
      $algorithm
    );

    $this->assertNotEquals($signature1, $signature2);
  }

  /**
   * Tests that special characters in inputs are handled correctly.
   *
   * @covers ::generateSignature
   */
  public function testSpecialCharactersInInputs(): void {
    $host = 'example.com';
    $uri = '/files/test file with spaces.pdf';
    $date = '2024-03-20';
    $expires = 3600;
    $algorithm = 'sha256';

    $signature = $this->signatureGenerator->generateSignature(
      $host,
      $uri,
      $date,
      $expires,
      $algorithm
    );

    // Test that the signature is still a valid SHA-256 hash.
    $this->assertEquals(64, strlen($signature));
    $this->assertTrue(ctype_xdigit($signature));

    // Test with URI containing special characters.
    $specialUri = '/files/test!@#$%^&*().pdf';
    $specialSignature = $this->signatureGenerator->generateSignature(
      $host,
      $specialUri,
      $date,
      $expires,
      $algorithm
    );

    $this->assertEquals(64, strlen($specialSignature));
    $this->assertTrue(ctype_xdigit($specialSignature));
    $this->assertNotEquals($signature, $specialSignature);
  }

}
